@@ -1,37 +1,24 @@
 /*
- *   Copyright 2006-2007 by Aaron Seigo <aseigo@kde.org>
- *   Copyright 2007 by Riccardo Iaconelli <riccardo@kde.org>
- *   Copyright 2008 by Ménard Alexis <darktears31@gmail.com>
+    SPDX-FileCopyrightText: 2006-2007 Aaron Seigo <aseigo@kde.org>
+    SPDX-FileCopyrightText: 2007 Riccardo Iaconelli <riccardo@kde.org>
+    SPDX-FileCopyrightText: 2008 Ménard Alexis <darktears31@gmail.com>
 
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU Library General Public License as
- *   published by the Free Software Foundation; either version 2, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details
- *
- *   You should have received a copy of the GNU Library General Public
- *   License along with this program; if not, write to the
- *   Free Software Foundation, Inc.,
- *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
- */
+    SPDX-License-Identifier: LGPL-2.0-or-later
+*/
 
 #ifndef PLASMA_APPLET_H
 #define PLASMA_APPLET_H
 
-#include <QObject>
 #include <QKeySequence>
+#include <QObject>
 #include <QUrl>
 
-#include <kconfiggroup.h>
-#include <kplugininfo.h>
+#include <KConfigGroup>
+#include <KPluginInfo>
 
+#include <plasma/framesvg.h>
 #include <plasma/plasma.h>
 #include <plasma/version.h>
-#include <plasma/framesvg.h>
 
 #include <KPackage/Package>
 
@@ -40,7 +27,6 @@ class KConfigLoader;
 
 namespace Plasma
 {
-
 class AppletPrivate;
 class Containment;
 class DataEngine;
@@ -71,11 +57,28 @@ class PLASMA_EXPORT Applet : public QObject
     Q_PROPERTY(Plasma::Types::ImmutabilityType immutability READ immutability WRITE setImmutability NOTIFY immutabilityChanged)
     Q_PROPERTY(Plasma::Types::FormFactor formFactor READ formFactor NOTIFY formFactorChanged)
     Q_PROPERTY(Plasma::Types::Location location READ location NOTIFY locationChanged)
-    Q_PROPERTY(QString title READ title WRITE setTitle FINAL)
-    Q_PROPERTY(QString icon READ icon WRITE setIcon FINAL)
+    Q_PROPERTY(Plasma::Types::ContainmentDisplayHints containmentDisplayHints READ containmentDisplayHints NOTIFY containmentDisplayHintsChanged)
+    Q_PROPERTY(QString title READ title WRITE setTitle NOTIFY titleChanged FINAL)
+    Q_PROPERTY(QString icon READ icon WRITE setIcon NOTIFY iconChanged FINAL)
+    Q_PROPERTY(bool busy READ isBusy WRITE setBusy NOTIFY busyChanged FINAL)
+
+    /**
+     * How the applet wants its background to be drawn. The containment may chose to ignore this hint.
+     */
+    Q_PROPERTY(Plasma::Types::BackgroundHints backgroundHints WRITE setBackgroundHints READ backgroundHints NOTIFY backgroundHintsChanged FINAL)
+
+    /**
+     * The containment (and/or the user) may decide to use another kind of background instead (if supported by the applet)
+     */
+    Q_PROPERTY(Plasma::Types::BackgroundHints userBackgroundHints WRITE setUserBackgroundHints READ userBackgroundHints NOTIFY userBackgroundHintsChanged FINAL)
+
+    /**
+     * The effective background hints the applet has, internally decided how to mix with userBackgroundHints
+     */
+    Q_PROPERTY(Plasma::Types::BackgroundHints effectiveBackgroundHints READ effectiveBackgroundHints NOTIFY effectiveBackgroundHintsChanged FINAL)
 
 public:
-//CONSTRUCTORS
+    // CONSTRUCTORS
     /**
      * @param parent the QObject this applet is parented to
      * @param serviceId the name of the .desktop file containing the
@@ -83,20 +86,34 @@ public:
      * @param appletId a unique id used to differentiate between multiple
      *      instances of the same Applet type
      */
-    explicit Applet(QObject *parent = 0, const QString &serviceId = QString(), uint appletId = 0);
+    explicit Applet(QObject *parent = nullptr, const QString &serviceId = QString(), uint appletId = 0);
 
+#if PLASMA_ENABLE_DEPRECATED_SINCE(5, 28)
     /**
      * @param parent the QObject this applet is parented to
      * @param info the plugin information object for this Applet
      * @param appletId a unique id used to differentiate between multiple
      *      instances of the same Applet type
      * @since 4.6
+     *
+     * @deprecated Since 5.28, prefer using KPluginMetaData
      */
-    explicit Applet(const KPluginInfo &info, QObject *parent = 0, uint appletId = 0);
+    PLASMA_DEPRECATED_VERSION(5, 28, "Use Applet(const KPluginMetaData &, QObject *, uint")
+    explicit Applet(const KPluginInfo &info, QObject *parent = nullptr, uint appletId = 0);
+#endif
 
-    ~Applet();
+    /**
+     * @param parent the QObject this applet is parented to
+     * @param metadata the plugin information object for this Applet
+     * @param appletId a unique id used to differentiate between multiple
+     *      instances of the same Applet type
+     * @since 5.27
+     */
+    explicit Applet(const KPluginMetaData &metadata, QObject *parent = nullptr, uint appletId = 0);
 
-//BOOKKEEPING
+    ~Applet() override;
+
+    // BOOKKEEPING
     /**
      * @return the id of this applet
      */
@@ -158,14 +175,20 @@ public:
      */
     Types::Location location() const;
 
-//CONFIGURATION
     /**
-    * Returns the KConfigGroup to access the applets configuration.
-    *
-    * This config object will write to an instance
-    * specific config file named \<appletname\>\<instanceid\>rc
-    * in the Plasma appdata directory.
-    **/
+     * @return Display hints that come from the containment that suggest the applet how to look and behave.
+     * @since 5.77
+     */
+    Types::ContainmentDisplayHints containmentDisplayHints() const;
+
+    // CONFIGURATION
+    /**
+     * Returns the KConfigGroup to access the applets configuration.
+     *
+     * This config object will write to an instance
+     * specific config file named \<appletname\>\<instanceid\>rc
+     * in the Plasma appdata directory.
+     **/
     KConfigGroup config() const;
 
     /**
@@ -235,17 +258,17 @@ public:
      */
     void setUserConfiguring(bool configuring);
 
-//UTILS
-#ifndef PLASMA_NO_DEPRECATED
+// UTILS
+#if PLASMA_ENABLE_DEPRECATED_SINCE(5, 6)
     /**
      * Accessor for the associated Package object if any.
      * Generally, only Plasmoids come in a Package.
-     * Deprecated: please use kPackage()
      *
-     * @deprecated use kPackage() instead
      * @return the Package object, or an invalid one if none
+     * @deprecated Since 5.6, use kPackage() instead
      **/
-    PLASMA_DEPRECATED Package package() const;
+    PLASMA_DEPRECATED_VERSION(5, 6, "Use Applet::kPackage()")
+    Package package() const;
 #endif
 
     /**
@@ -266,13 +289,24 @@ public:
      */
     void updateConstraints(Plasma::Types::Constraints constraints = Plasma::Types::AllConstraints);
 
-//METADATA
+// METADATA
+#if PLASMA_ENABLE_DEPRECATED_SINCE(5, 28)
     /**
      * @return metadata information about this plugin
-     * @see KPluginInfo
+     * @see KPluginInfo, pluginMetaData
      * @since 5.0
+     * @deprecated Since 5.28, use pluginMetaData instead
      */
+    PLASMA_DEPRECATED_VERSION(5, 28, "Use Applet::pluginMetaData()")
     KPluginInfo pluginInfo() const;
+#endif
+
+    /**
+     * @return metadata information about this plugin
+     *
+     * @since 5.27
+     */
+    KPluginMetaData pluginMetaData() const;
 
     /**
      * Returns the user-visible title for the applet, as specified in the
@@ -290,16 +324,15 @@ public:
      * @since 5.0
      * @param title the user-visible title for the applet.
      */
-    void setTitle(const QString &title) const;
+    void setTitle(const QString &title);
 
+#if PLASMA_ENABLE_DEPRECATED_SINCE(5, 19)
     /**
      * Attempts to load an applet from a package
      *
      * Returns a pointer to the applet if successful.
      * The caller takes responsibility for the applet, including
      * deleting it when no longer needed.
-     * Deprecated: use Containment::createApplet instead, you are not
-     * supposed to have applets without containments
      * If you instance a plasmoid with this deprecated API, the
      * automated default setup scripts won't be executed for that plasmoid
      *
@@ -308,8 +341,13 @@ public:
      *        assigned automatically.
      * @return a pointer to the loaded applet, or 0 on load failure
      * @since 4.3
+     *
+     * @deprecated Since 5.19, use Containment::createApplet() instead, you are not
+     * supposed to have applets without containments
      **/
-    PLASMA_DEPRECATED static Applet *loadPlasmoid(const QString &path, uint appletId = 0);
+    PLASMA_DEPRECATED_VERSION(5, 19, "Use Containment::createApplet(...)")
+    static Applet *loadPlasmoid(const QString &path, uint appletId = 0);
+#endif
 
     /**
      * @returns The icon name related to this applet
@@ -323,7 +361,57 @@ public:
      */
     void setIcon(const QString &icon);
 
-//ACTIONS
+    /**
+     * @returns true if the applet should show a busy status, for instance doing
+     * some network operation
+     * @since 5.21
+     */
+    bool isBusy() const;
+
+    /**
+     * Sets the Applet to have a busy status hint, for instance the applet doing
+     * some network operation.
+     * The graphical representation of the busy status depends completely from
+     * the visualization.
+     * @param busy true if the applet is busy
+     * @since 5.21
+     */
+    void setBusy(bool busy);
+
+    /**
+     * How the applet wants its background to be drawn. The containment may chose to ignore this hint.
+     * @since 5.65
+     */
+    Plasma::Types::BackgroundHints backgroundHints() const;
+
+    /**
+     * Sets the applet background hints. Only Applet implementations should write this property
+     * @since 5.65
+     */
+    void setBackgroundHints(Plasma::Types::BackgroundHints hint);
+
+    /**
+     * The containment (and/or the user) may decide to use another kind of background instead if supported by the applet.
+     * In order for an applet to support user configuration of the
+     * background, it needs to have the Plasma::Types::ConfigurableBackground flag set in its backgroundHints
+     * @since 5.65
+     */
+    Plasma::Types::BackgroundHints userBackgroundHints() const;
+
+    /**
+     * Sets the hints the user wished the background style for the applet to be.
+     * @since 5.65
+     */
+    void setUserBackgroundHints(Plasma::Types::BackgroundHints hint);
+
+    /**
+     * The effective background hints the applet will have: it will follow userBackgroundHints only if backgroundHints has the
+     * Plasma::Types::ConfigurableBackground flag set
+     * @since 5.65
+     */
+    Plasma::Types::BackgroundHints effectiveBackgroundHints() const;
+
+    // ACTIONS
     /**
      * Returns a list of context-related QAction instances.
      *
@@ -346,12 +434,12 @@ public:
     void setGlobalShortcut(const QKeySequence &shortcut);
 
     /**
-     * @return the global shortcut associated with this wiget, or
+     * @return the global shortcut associated with this widget, or
      * an empty shortcut if no global shortcut is associated.
      */
     QKeySequence globalShortcut() const;
 
-// ASSOCIATED APPLICATION
+    // ASSOCIATED APPLICATION
     /**
      * Sets an application associated to this applet, that will be
      * regarded as a full view of what is represented in the applet
@@ -392,14 +480,14 @@ public:
      */
     bool hasValidAssociatedApplication() const;
 
-//Completely UI-specific, remove or move to scriptengine
+    // Completely UI-specific, remove or move to scriptengine
     /**
      * @return true if this plasmoid provides a GUI configuration
      **/
     bool hasConfigurationInterface() const;
 
 Q_SIGNALS:
-//BOOKEEPING
+    // BOOKEEPING
     /**
      * Emitted when the immutability changes
      * @since 4.4
@@ -419,7 +507,43 @@ Q_SIGNALS:
      */
     void destroyedChanged(bool destroyed);
 
-//CONFIGURATION
+    /**
+     * Emitted when the title has changed
+     * @since 5.20
+     */
+    void titleChanged(const QString &title);
+
+    /**
+     * Emitted when the icon name for the applet has changed
+     * @since 5.20
+     */
+    void iconChanged(const QString &icon);
+
+    /**
+     * Emitted when the busy status has changed
+     * @since 5.21
+     */
+    void busyChanged(bool busy);
+
+    /**
+     * Emitted when the background hints have changed
+     * @since 5.65
+     */
+    void backgroundHintsChanged();
+
+    /**
+     * Emitted when the user background hints have changed
+     * @since 5.65
+     */
+    void userBackgroundHintsChanged();
+
+    /**
+     * Emitted when the effective background hints have changed
+     * @since 5.65
+     */
+    void effectiveBackgroundHintsChanged();
+
+    // CONFIGURATION
     /**
      * Emitted when an applet has changed values in its configuration
      * and wishes for them to be saved at the next save point. As this implies
@@ -435,7 +559,7 @@ Q_SIGNALS:
      */
     void userConfiguringChanged(bool configuring);
 
-//ACTIONS
+    // ACTIONS
     /**
      * Emitted just before the contextual actions are about to show
      * For instance just before the context menu containing the actions
@@ -445,11 +569,11 @@ Q_SIGNALS:
 
     /**
      * Emitted when activation is requested due to, for example, a global
-     * keyboard shortcut. By default the wiget is given focus.
+     * keyboard shortcut. By default the widget is given focus.
      */
     void activated();
 
-//TODO: fix usage in containment, port to QObject::destroyed
+    // TODO: fix usage in containment, port to QObject::destroyed
     /**
      * Emitted when the applet is deleted
      */
@@ -465,6 +589,8 @@ Q_SIGNALS:
      */
     void locationChanged(Plasma::Types::Location location);
 
+    void containmentDisplayHintsChanged(Plasma::Types::ContainmentDisplayHints hints);
+
     /**
      * Emitted when setConfigurationRequired was called
      * @see setConfigurationRequired
@@ -473,7 +599,7 @@ Q_SIGNALS:
     void configurationRequiredChanged(bool needsConfig, const QString &reason);
 
 public Q_SLOTS:
-//BOOKKEEPING
+    // BOOKKEEPING
     /**
      * Call this method when the applet fails to launch properly. An
      * optional reason can be provided.
@@ -509,13 +635,14 @@ public Q_SLOTS:
      */
     void setStatus(const Types::ItemStatus stat);
 
-//CONFIGURATION
+    // CONFIGURATION
     /**
      * Called when applet configuration values have changed.
      */
+    // TODO KF6: make it not a slot anymore and protected
     virtual void configChanged();
 
-//UTILS
+    // UTILS
     /**
      * Sends all pending constraints updates to the applet. Will usually
      * be called automatically, but can also be called manually if needed.
@@ -534,19 +661,19 @@ public Q_SLOTS:
      **/
     virtual void init();
 
-//ASSOCIATED APPLICATION
+    // ASSOCIATED APPLICATION
     /**
-    * Open the application associated to this applet, if it's not set
-    * but some urls are, open those urls with the proper application
-    * for their mimetype
-    * @see setAssociatedApplication()
-    * @see setAssociatedApplicationUrls()
-    * @since 4.4
-    */
+     * Open the application associated to this applet, if it's not set
+     * but some urls are, open those urls with the proper application
+     * for their mimetype
+     * @see setAssociatedApplication()
+     * @see setAssociatedApplicationUrls()
+     * @since 4.4
+     */
     void runAssociatedApplication();
 
 protected:
-//CONSTRUCTORS
+    // CONSTRUCTORS
     /**
      * This constructor is to be used with the plugin loading systems
      * found in KPluginInfo and KService. The argument list is expected
@@ -559,7 +686,7 @@ protected:
      */
     Applet(QObject *parent, const QVariantList &args);
 
-//CONFIGURATION
+    // CONFIGURATION
     /**
      * When called, the Applet should write any information needed as part
      * of the Applet's running state to the configuration object in config()
@@ -595,7 +722,7 @@ protected:
      */
     void setConfigurationRequired(bool needsConfiguring, const QString &reason = QString());
 
-//UTILS
+    // UTILS
     /**
      * Called when any of the constraints for the applet have been updated. These constraints
      * range from notifying when the applet has officially "started up" to when geometry changes
@@ -616,11 +743,11 @@ protected:
      */
     virtual void constraintsEvent(Plasma::Types::Constraints constraints);
 
-//TODO: timerEvent should go into AppletPrivate
+    // TODO: timerEvent should go into AppletPrivate
     /**
      * Reimplemented from QObject
      */
-    void timerEvent(QTimerEvent *event) Q_DECL_OVERRIDE;
+    void timerEvent(QTimerEvent *event) override;
 
 private:
     /**
@@ -642,7 +769,7 @@ private:
 
     AppletPrivate *const d;
 
-    //Corona needs to access setLaunchErrorMessage and init
+    // Corona needs to access setLaunchErrorMessage and init
     friend class Corona;
     friend class CoronaPrivate;
     friend class Containment;
@@ -660,12 +787,12 @@ private:
 /**
  * Register an applet when it is contained in a loadable module
  */
+/* clang-format off */
 #define K_EXPORT_PLASMA_APPLET(libname, classname) \
-    K_PLUGIN_FACTORY(factory, registerPlugin<classname>();) \
-    K_EXPORT_PLUGIN_VERSION(PLASMA_VERSION)
+    K_PLUGIN_FACTORY(factory, registerPlugin<classname>();)
 
 #define K_EXPORT_PLASMA_APPLET_WITH_JSON(libname, classname, jsonFile) \
-    K_PLUGIN_FACTORY_WITH_JSON(factory, jsonFile, registerPlugin<classname>();) \
-    K_EXPORT_PLUGIN_VERSION(PLASMA_VERSION)
+    K_PLUGIN_FACTORY_WITH_JSON(factory, jsonFile, registerPlugin<classname>();)
+/* clang-format on */
 
 #endif // multiple inclusion guard
